
#include "xparameters.h"
#include "stdio.h"
#include "sleep.h"
#include "radio_controller_basic.h"
#include "radio_controller_ext.h"
#include "radio_controller_adv.h"
#include "radio_controller_5ghz.h"
#include "EEPROM.h"
#include "warp_v4_userio.h"
#include "xgpio.h"

#define USERIO_BASEADDR XPAR_USERIO_BASEADDR
#define CAPTURE_PERIOD		XPAR_PRNG_USERIOSRC_PLBW_0_MEMMAP_CAPTUREPERIOD
#define CAPTURED_OUTPUT		XPAR_PRNG_USERIOSRC_PLBW_0_MEMMAP_CAPTUREDOUTPUT

void userio_init();
int radio_init();
void userio_example();

//Assume a 2-radio kit, with WARP Radio Boards v1.4 in slots 2 and 3
unsigned int radios = RADIO2_ADDR | RADIO3_ADDR;

int main() {

	print("\f");
	print("WARP v2 Template Project - Example Application\n");

	userio_init();
	radio_init();
	userio_example();

	return 0;
}


//Simple demonstration of the user IO on FPGA Board v2
// This function runs forever, reading the user inputs (DIP switch and push buttons)
// and updating the user outputs (hex displays and LEDs) based on the inputs
//Remember the "down" button (SW6) is tied to the PPC reset
void userio_example() {

	unsigned int assertedInputs;

	u16 capturedValue;
	u8 i,allowPrint;

		/*
		 * We use allowPrint as a simple software "latch" to avoid unnecessary prints to the UART.
		 * When the user presses and holds the "up" pushbutton on the board, this code will print
		 * the binary string of the output of the custom core's LFSR. When the user releases the
		 * pushbutton, this code will print a series of backspaces to delete the previous print.
		 */
		allowPrint = 1;

		/*
		 * The CAPTURE_PERIOD definition simply abstracts away the address of the "capturePeriod"
		 * register that was created in the underlying custom pcore. XIo_Out32 is a simple macro
		 * that Xilinx provides that lets us write values to that memory address. In this function,
		 * we write a value that will ensure that the USER I/O LEDs are updated at 10 times per second.
		 * Reduce this value to increase the speed of the updates and increase it to slow it down further.
		 */
		XIo_Out32(CAPTURE_PERIOD,8000000-1);

		xil_printf("\t            Red LEDs  -------                 -------  Green LEDs\n");
		xil_printf("\tPRNG Captured Value:                                  ");

		/*
		 * This while loop will run forever. There is no exit condition to break out of this loop.
		 * This loop will continually read from the User I/O core to determine if any pushbuttons
		 * are currently being pressed. If so, it triggers a small loop to print out the output
		 * of the custom pcore's capturedOutput register. This inner loop is simply to display
		 * the output as a binary string.
		 */
		while(1){

			//Read the GPIO inputs; each 1 is a currently-asserted input bit
			assertedInputs = WarpV4_UserIO_PushB(USERIO_BASEADDR) | (WarpV4_UserIO_DipSw(USERIO_BASEADDR) << 4);

			if(assertedInputs & USERIO_MASK_PBU){
				if(allowPrint){
					capturedValue = XIo_In32(CAPTURED_OUTPUT);
					xil_printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
					for (i=0;i<16;i++){
						xil_printf("%d ", (capturedValue>>(15-i))&1);
					}
					allowPrint = 0;
				}
			} else if(!allowPrint) {
				xil_printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
				xil_printf("                                ");
				allowPrint = 1;
			}
		}
		xil_printf("\n");
		return;
}

void userio_init() {

	//Set all hex displays to auto-map mode
	WarpV4_UserIO_NumberMode_LeftHex(USERIO_BASEADDR);
	WarpV4_UserIO_NumberMode_MiddleHex(USERIO_BASEADDR);
	WarpV4_UserIO_NumberMode_RightHex(USERIO_BASEADDR);

	//Set all hex displays to show "0"
	WarpV4_UserIO_WriteNumber_LeftHex(USERIO_BASEADDR, 0, 0);
	WarpV4_UserIO_WriteNumber_MiddleHex(USERIO_BASEADDR, 0, 0);
	WarpV4_UserIO_WriteNumber_RightHex(USERIO_BASEADDR, 0, 0);

	//Turn off all user LEDs
	WarpV4_UserIO_Leds(USERIO_BASEADDR, 0);

}

int radio_init() {
	//Initialize the radio transceivers
	WarpRadio_v1_Reset((unsigned int*)XPAR_RADIO_CONTROLLER_0_BASEADDR, 2);

	//Set some sane defaults for the MAX2829 transceivers
	WarpRadio_v1_SetCenterFreq2GHz(11, radios);
	WarpRadio_v1_SoftwareTxGainControl(1, radios);
	WarpRadio_v1_SoftwareRxGainControl(1, radios);
	WarpRadio_v1_RxHpSoftControlEnable(radios);
	WarpRadio_v1_RxLpfCornFreqCoarseAdj(1, radios);
	WarpRadio_v1_TxLpfCornFreqCoarseAdj(1, radios);

	//Ensure both transceivers are in standby (no Tx or Rx)
	WarpRadio_v1_TxRxDisable(radios);

	return 0;
}
